@@ -148,8 +148,8 @@ int main(int argc, char *argv[])
 	/* Initiates DEBUG - but no debugging here ! */
 
 static int static_get_options(argc,argv)
-int *argc;
-char **argv[];
+register int *argc;
+register char **argv[];
 {
   int help,version;
   char *pos;
@@ -173,7 +173,6 @@ char **argv[];
 	break;
       case 'V':
 	version=1;
-      // fallthrough
       case 'I':
       case '?':
 	help=1;					/* Help text written */
@@ -218,14 +217,14 @@ char **argv[];
 
 
 static int get_replace_strings(argc,argv,from_array,to_array)
-int *argc;
-char **argv[];
+register int *argc;
+register char **argv[];
 POINTER_ARRAY *from_array,*to_array;
 {
   char *pos;
 
-  bzero((char*) from_array,sizeof(from_array[0]));
-  bzero((char*) to_array,sizeof(to_array[0]));
+  memset(from_array, 0, sizeof(from_array[0]));
+  memset(to_array, 0, sizeof(to_array[0]));
   while (*argc > 0 && (*(pos = *(*argv)) != '-' || pos[1] != '-' || pos[2]))
   {
     insert_pointer_name(from_array,pos);
@@ -248,7 +247,7 @@ POINTER_ARRAY *from_array,*to_array;
   return 0;
 }
 
-static int insert_pointer_name(POINTER_ARRAY *pa,char * name)
+static int insert_pointer_name(reg1 POINTER_ARRAY *pa,char * name)
 {
   uint i,length,old_count;
   uchar *new_pos;
@@ -322,7 +321,7 @@ static int insert_pointer_name(POINTER_ARRAY *pa,char * name)
 
 	/* free pointer array */
 
-static void free_pointer_array(POINTER_ARRAY *pa)
+static void free_pointer_array(reg1 POINTER_ARRAY *pa)
 {
   if (pa->typelib.count)
   {
@@ -423,7 +422,7 @@ static REPLACE *init_replace(char * *from, char * *to,uint count,
     if (len > max_length)
       max_length=len;
   }
-  bzero((char*) is_word_end,sizeof(is_word_end));
+  memset(is_word_end, 0, sizeof(is_word_end));
   for (i=0 ; word_end_chars[i] ; i++)
     is_word_end[(uchar) word_end_chars[i]]=1;
 
@@ -542,7 +541,7 @@ static REPLACE *init_replace(char * *from, char * *to,uint count,
       or_bits(sets.set+used_sets,sets.set);	/* Can restart from start */
 
     /* Find all chars that follows current sets */
-    bzero((char*) used_chars,sizeof(used_chars));
+    memset(used_chars, 0, sizeof(used_chars));
     for (i= (uint) ~0; (i=get_next_bit(sets.set+used_sets,i)) ;)
     {
       used_chars[follow[i].chr]=1;
@@ -676,7 +675,7 @@ static REPLACE *init_replace(char * *from, char * *to,uint count,
 
 static int init_sets(REP_SETS *sets,uint states)
 {
-  bzero((char*) sets,sizeof(*sets));
+  memset(sets, 0, sizeof(*sets));
   sets->size_of_bits=((states+7)/8);
   if (!(sets->set_buffer=(REP_SET*) my_malloc(sizeof(REP_SET)*SET_MALLOC_HUNC,
 					      MYF(MY_WME))))
@@ -707,8 +706,8 @@ static REP_SET *make_new_set(REP_SETS *sets)
   {
     sets->extra--;
     set=sets->set+ sets->count++;
-    bzero((char*) set->bits,sizeof(uint)*sets->size_of_bits);
-    bzero((char*) &set->next[0],sizeof(set->next[0])*LAST_CHAR_CODE);
+    memset(set->bits, 0, sizeof(uint)*sets->size_of_bits);
+    memset(&set->next[0], 0, sizeof(set->next[0])*LAST_CHAR_CODE);
     set->found_offset=0;
     set->found_len=0;
     set->table_offset= (uint) ~0;
@@ -765,7 +764,7 @@ static void internal_clear_bit(REP_SET *set, uint bit)
 
 static void or_bits(REP_SET *to,REP_SET *from)
 {
-  uint i;
+  reg1 uint i;
   for (i=0 ; i < to->size_of_bits ; i++)
     to->bits[i]|=from->bits[i];
   return;
@@ -883,8 +882,8 @@ static uint replace_len(char * str)
 static uint replace_strings(REPLACE *rep, char **start, uint *max_length,
                             char *from)
 {
-  REPLACE *rep_pos;
-  REPLACE_STRING *rep_str;
+  reg1 REPLACE *rep_pos;
+  reg2 REPLACE_STRING *rep_str;
   char *to, *end, *pos, *new;
 
   end=(to= *start) + *max_length-1;
