@@ -18,6 +18,7 @@ niceness=0
 mysqld_ld_preload=
 mysqld_ld_library_path=
 load_jemalloc=1
+load_hotbackup=0
 flush_caches=0
 numa_interleave=0
 # Change (disable) transparent huge pages (TokuDB requirement)
@@ -245,6 +246,7 @@ parse_arguments() {
       --open_files_limit=*) open_files="$val" ;;
       --skip-kill-mysqld*) KILL_MYSQLD=0 ;;
       --thp-setting=*) thp_setting="$val" ;;
+      --preload-hotbackup) load_hotbackup=1 ;;
       --syslog) want_syslog=1 ;;
       --skip-syslog) want_syslog=0 ;;
       --syslog-tag=*) syslog_tag="$val" ;;
@@ -524,7 +526,7 @@ parse_arguments PICK-ARGS-FROM-ARGV "$@"
 #
 if test $load_jemalloc -eq 1
 then
-  for libjemall in "/usr/lib64" "/usr/lib/x86_64-linux-gnu" "/usr/lib"; do
+  for libjemall in "${MY_BASEDIR_VERSION}/lib/mysql" "/usr/lib64" "/usr/lib/x86_64-linux-gnu" "/usr/lib"; do
     if [ -r "$libjemall/libjemalloc.so.1" ]; then
       add_mysqld_ld_preload "$libjemall/libjemalloc.so.1"
       break
@@ -532,6 +534,19 @@ then
   done
 fi
 
+#
+# Add TokuDB HotBackup library to ld_preload
+#
+if test $load_hotbackup -eq 1
+then
+  for libhb in "${MY_BASEDIR_VERSION}/lib" "/usr/lib64" "/usr/lib/x86_64-linux-gnu" "/usr/lib"; do
+    if [ -r "$libhb/libHotBackup.so" ]; then
+      add_mysqld_ld_preload "$libhb/libHotBackup.so"
+      break
+    fi
+  done
+fi
+
 #
 # Try to find the plugin directory
 #
