@@ -1,4 +1,4 @@
-/* Copyright (c) 2005, 2014, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2005, 2015, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -490,6 +490,7 @@ public:
   volatile ulong pending_jobs;
   mysql_mutex_t pending_jobs_lock;
   mysql_cond_t pending_jobs_cond;
+  mysql_mutex_t exit_count_lock; // mutex of worker exit count
   ulong       mts_slave_worker_queue_len_max;
   ulonglong   mts_pending_jobs_size;      // actual mem usage by WQ:s
   ulonglong   mts_pending_jobs_size_max;  // max of WQ:s size forcing C to wait
@@ -525,9 +526,11 @@ public:
   ulong mts_coordinator_basic_nap; // C sleeps to avoid WQs overrun
   ulong opt_slave_parallel_workers; // cache for ::opt_slave_parallel_workers
   ulong slave_parallel_workers; // the one slave session time number of workers
+  ulong exit_counter; // Number of workers contributed to max updated group index
+  ulonglong max_updated_index;
   ulong recovery_parallel_workers; // number of workers while recovering
   uint checkpoint_seqno;  // counter of groups executed after the most recent CP
-  uint checkpoint_group;  // cache for ::opt_mts_checkpoint_group 
+  uint checkpoint_group;  // cache for ::opt_mts_checkpoint_group
   MY_BITMAP recovery_groups;  // bitmap used during recovery
   bool recovery_groups_inited;
   ulong mts_recovery_group_cnt; // number of groups to execute at recovery
@@ -562,10 +565,10 @@ public:
   } mts_group_status;
 
   /*
-    MTS statistics: 
+    MTS statistics:
   */
   ulonglong mts_events_assigned; // number of events (statements) scheduled
-  ulong mts_groups_assigned; // number of groups (transactions) scheduled
+  ulonglong mts_groups_assigned; // number of groups (transactions) scheduled
   volatile ulong mts_wq_overrun_cnt; // counter of all mts_wq_excess_cnt increments
   ulong wq_size_waits_cnt;    // number of times C slept due to WQ:s oversize
   /*
@@ -599,6 +602,16 @@ public:
     return mts_recovery_group_cnt != 0;
   }
 
+  inline void clear_mts_recovery_groups()
+  {
+    if (recovery_groups_inited)
+    {
+      bitmap_free(&recovery_groups);
+      mts_recovery_group_cnt= 0;
+      recovery_groups_inited= false;
+    }
+  }
+
   /**
      returns true if events are to be executed in parallel
   */
