@@ -61,6 +61,10 @@ void vio_set_wait_callback(void (*before_wait)(void),
   after_io_wait= after_wait;
 }
 
+/* Array of networks which have the proxy protocol activated */
+static struct st_vio_network *vio_pp_networks= NULL;
+static size_t vio_pp_networks_nb= 0;
+
 int vio_errno(Vio *vio __attribute__((unused)))
 {
   /* These transport types are not Winsock based. */
@@ -669,6 +673,260 @@ my_bool vio_get_normalized_ip_string(const struct sockaddr *addr,
 }
 
 
+/* Add a network to the proxied network list. */
+void vio_proxy_protocol_add(const struct st_vio_network *net)
+{
+  /* Grow the vio_pp_networks array. Calling realloc for every single element
+  is not particularly efficient, but this is done once per server startup with
+  relatively few allowed networks. */
+  vio_pp_networks_nb++;
+  vio_pp_networks= my_realloc(vio_pp_networks,
+                              vio_pp_networks_nb * sizeof(*net),
+                              MYF(MY_ALLOW_ZERO_PTR | MY_FAE | MY_WME));
+  memcpy(&vio_pp_networks[vio_pp_networks_nb - 1], net, sizeof(*net));
+}
+
+/* Check whether a connection from this source address must provide the proxy
+protocol header */
+static my_bool vio_client_must_be_proxied(const struct sockaddr *addr)
+{
+  size_t i;
+  for (i= 0; i < vio_pp_networks_nb; i++)
+    if (vio_pp_networks[i].family == addr->sa_family) {
+      if (vio_pp_networks[i].family == AF_INET) {
+        struct in_addr *check= &((struct sockaddr_in *)addr)->sin_addr;
+        struct in_addr *addr= &vio_pp_networks[i].addr.in;
+        struct in_addr *mask= &vio_pp_networks[i].mask.in;
+        if ((check->s_addr & mask->s_addr) == addr->s_addr)
+          return TRUE;
+      }
+#ifdef HAVE_IPV6
+      else {
+        struct in6_addr *check= &((struct sockaddr_in6 *)addr)->sin6_addr;
+        struct in6_addr *addr= &vio_pp_networks[i].addr.in6;
+        struct in6_addr *mask= &vio_pp_networks[i].mask.in6;
+        DBUG_ASSERT(vio_pp_networks[i].family == AF_INET6);
+        if ((check->s6_addr32[0] & mask->s6_addr32[0]) == addr->s6_addr32[0]
+            && ((check->s6_addr32[1] & mask->s6_addr32[1])
+                == addr->s6_addr32[1])
+            && ((check->s6_addr32[2] & mask->s6_addr32[2])
+                == addr->s6_addr32[2])
+            && ((check->s6_addr32[3] & mask->s6_addr32[3])
+                == addr->s6_addr32[3]))
+          return TRUE;
+      }
+#endif
+    }
+  return FALSE;
+}
+
+/* Process the proxy protocol header. Return true on an error. */
+static my_bool vio_process_proxy_header(int socket_fd, struct sockaddr *addr,
+                                        size_socket *addr_length)
+{
+  /* The ip source network matches an expected proxy protocol network. */
+  static const char v2sig[12]=
+    "\x0D\x0A\x0D\x0A\x00\x0D\x0A\x51\x55\x49\x54\x0A";
+  union {
+    struct {
+      char line[108];
+    } v1;
+    struct {
+      uint8_t sig[12];
+      uint8_t ver_cmd;
+      uint8_t fam;
+      uint16_t len;
+      union {
+        struct {  /* for TCP/UDP over IPv4, len = 12 */
+          uint32_t src_addr;
+          uint32_t dst_addr;
+          uint16_t src_port;
+          uint16_t dst_port;
+        } __attribute__((packed)) ip4;
+#ifdef HAVE_IPV6
+        struct {  /* for TCP/UDP over IPv6, len = 36 */
+          uint8_t  src_addr[16];
+          uint8_t  dst_addr[16];
+          uint16_t src_port;
+          uint16_t dst_port;
+        } __attribute__((packed)) ip6;
+#endif
+      } addr;
+    } __attribute__((packed)) v2;
+  } hdr;
+
+  int size;
+  struct sockaddr_storage from;
+  int from_len;
+  ssize_t ret;
+
+  do {
+    ret= recv(socket_fd, &hdr, sizeof(hdr), MSG_PEEK);
+  } while (ret == -1 && errno == EINTR);
+
+  /* if the recv returns an error, the proxy protocol is ignored. */
+  if (ret == -1)
+    return TRUE;
+
+  memset(&from, 0x00, sizeof(struct sockaddr_storage));
+
+  if (ret >= 16 && memcmp(&hdr.v2, v2sig, 12) == 0 &&
+      (hdr.v2.ver_cmd & 0xF0) == 0x20) {
+
+    /* proxy-protocool v2. */
+
+    size= 16 + ntohs(hdr.v2.len);
+
+    /* truncated or too large header */
+    if (ret < size)
+      return TRUE;
+
+    switch (hdr.v2.ver_cmd & 0xF) {
+    case 0x01: /* PROXY command */
+      switch (hdr.v2.fam) {
+      case 0x11:  /* TCPv4 */
+        ((struct sockaddr_in *)&from)->sin_family= AF_INET;
+        ((struct sockaddr_in *)&from)->sin_addr.s_addr=
+          hdr.v2.addr.ip4.src_addr;
+        ((struct sockaddr_in *)&from)->sin_port=
+          hdr.v2.addr.ip4.src_port;
+        from_len= sizeof(struct sockaddr_in);
+        goto pp_done;
+#ifdef HAVE_IPV6
+      case 0x21:  /* TCPv6 */
+        ((struct sockaddr_in6 *)&from)->sin6_family= AF_INET6;
+        memcpy(&((struct sockaddr_in6 *)&from)->sin6_addr,
+               hdr.v2.addr.ip6.src_addr, 16);
+        ((struct sockaddr_in6 *)&from)->sin6_port=
+          hdr.v2.addr.ip6.src_port;
+        from_len= sizeof(struct sockaddr_in6);
+        goto pp_done;
+#endif
+      case 0x00: /* Unspec */
+        /* unknown protocol, keep local connection address */
+        goto pp_flush;
+      default:
+        return TRUE;
+      }
+      return TRUE;
+    case 0x00: /* LOCAL command */
+      /* keep local connection address for LOCAL */
+      goto pp_flush;
+    default:
+      /* not a supported command. Abort connexion */
+      return TRUE;
+    }
+
+    return TRUE;
+  }
+
+  if (ret >= 8 && memcmp(hdr.v1.line, "PROXY ", 6) == 0) {
+
+    /* proxy-protocol v1. */
+
+    int port;
+    char *p, *end= memchr(hdr.v1.line, '\r', ret - 1);
+    if (!end || *(end + 1) != '\n')
+      return TRUE; /* partial or invalid header */
+
+    *end= '\0'; /* terminate the string to ease parsing */
+    size= end + 2 - hdr.v1.line; /* skip header + CRLF */
+    /* parse the V1 header using favorite address parsers like inet_pton.
+     * return -1 upon error, or simply fall through to accept.
+     */
+    p= hdr.v1.line + strlen("PROXY ");
+    if (memcmp(p, "TCP4 ", 5) == 0) {
+      /* Parse IPv4. */
+      p+= strlen("TCP4 ");
+      end= strchr(p, ' ');
+      if (!end || end[0] != ' ')
+        return TRUE; /* malformatted pp. Abort connection. */
+      *end= '\0';
+      ((struct sockaddr_in *)&from)->sin_family= AF_INET;
+      if (!inet_pton(AF_INET, p, &((struct sockaddr_in *)&from)->sin_addr))
+        return TRUE; /* malformatted pp. Abort connection. */
+      from_len= sizeof(struct sockaddr_in);
+    }
+#ifdef HAVE_IPV6
+    else if (memcmp(p, "TCP6 ", 5) == 0) {
+      /* Parse IPv6. */
+      p+= strlen("TCP6 ");
+      end= strchr(p, ' ');
+      if (!end || end[0] != ' ')
+        return TRUE; /* malformatted pp. Abort connection. */
+      *end= '\0';
+      ((struct sockaddr_in6 *)&from)->sin6_family= AF_INET6;
+      if (!inet_pton(AF_INET6, p, &((struct sockaddr_in6 *)&from)->sin6_addr))
+        return TRUE; /* malformatted pp. Abort connection. */
+      from_len= sizeof(struct sockaddr_in6);
+    }
+#endif
+    else if (memcmp(p, "UNKNOWN", 7) == 0)
+      /* unknown protocol, keep local connection address */
+      goto pp_flush;
+
+    else
+      /* Unknown data, ignore the proxy protocol. */
+      return TRUE;
+
+    /* Check port. */
+    p= end + 1;
+    end= strchr(p, ' ');
+    if (!end || end[0] != ' ')
+      return TRUE; /* malformatted pp. Abort connection. */
+
+    p= end + 1;
+    end= strchr(p, ' ');
+    if (!end || end[0] != ' ')
+      return TRUE; /* malformatted pp. Abort connection. */
+
+    // FIXME: atoi here does not full protocol conformity validity (no
+    // leading zeros, sign, non-numeric characters etc)
+    *end= 0;
+    port= atoi(p);
+    if (port < 0 || port > 65535)
+      return TRUE; /* malformatted pp. Abort connection. */
+
+    if (from.ss_family == AF_INET)
+      ((struct sockaddr_in *)&from)->sin_port= htons((uint16_t)port);
+#ifdef HAVE_IPV6
+    if (from.ss_family == AF_INET6)
+      ((struct sockaddr_in6 *)&from)->sin6_port= htons((uint16_t)port);
+#endif
+  }
+  else {
+    /* Wrong protocol. Abort connection */
+    return TRUE;
+  }
+
+ pp_done:
+  /* Proxying localhost is forbidden */
+  if (from.ss_family == AF_INET
+      && (((struct sockaddr_in *)&from)->sin_addr.s_addr
+          == htonl(INADDR_LOOPBACK)))
+    return TRUE;
+#ifdef HAVE_IPV6
+  else if (from.ss_family == AF_INET6
+           && !memcmp(&((struct sockaddr_in6 *)&from)->sin6_addr,
+                      &in6addr_loopback, sizeof(struct in6_addr)))
+    return TRUE;
+#endif
+
+  /* Copy the decoded address. */
+  memcpy(addr, &from, from_len);
+  *addr_length= from_len;
+
+ pp_flush:
+  /* we need to consume the appropriate amount of data from the socket */
+  do {
+    ret= recv(socket_fd, &hdr, size, 0);
+  } while (ret == -1 && errno == EINTR);
+  if (ret == -1)
+    return TRUE;
+
+  return FALSE;
+}
+
 /**
   Return IP address and port of a VIO client socket.
 
@@ -723,6 +981,17 @@ my_bool vio_peer_addr(Vio *vio, char *ip_buffer, uint16 *port,
       DBUG_RETURN(TRUE);
     }
 
+    /* If the proxy protocol is activated for this listener and if the client
+       address is in a proxy protocol network, try to read proxy protocol and
+       determine the real source IP.
+
+       The proxy protocol source ip replace it the ip returned by
+       mysql_socket_getpeername(). */
+    if (vio_client_must_be_proxied(addr))
+      if (vio_process_proxy_header(mysql_socket_getfd(vio->mysql_socket), addr,
+                                   &addr_length))
+        DBUG_RETURN(TRUE);
+
     /* Normalize IP address. */
 
     vio_get_normalized_ip(addr, addr_length,
